/***************************************************************************/
/* wrap.c - Small program to wrap the input (text file) at a specified     */
/*          number of characters.                                          */
/***************************************************************************/
/*                    Copyright (c) 1997 Kevin Sage                        */
/*                        All Rights Reserved                              */
/***************************************************************************/
/* Rev. 1.00 08/09/97  K. Sage     Original Release                        */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*** Maximum string (input line) length ***/
#define MAX_STR_LEN 1024

/*** Version Information ***/
#define VERSION "1.00"

void usage(void);

int main(int argc, char* argv[])
{
	/*			  */
	/* Declare some variables */
	/*                        */
	char 
		sWrapAt[10],			/*** The number of columns to wrap at (in text) ***/
		InString[MAX_STR_LEN];		/*** The line we're reading in ***/
	int 
		x,				/*** A temporary counting variable ***/
		iWrapAt,			/*** The number of columns to wrap at (integer) ***/
		CurPos,				/*** The current character position ***/
		LastPos,			/*** The last index in the string ***/
		Offset;				/*** The multiple to offset position by ***/
	FILE* 
		filename;			/*** The file descriptor of the input file ***/

	/*                                           */
	/* Check that there is at least one argument */
	/*                                           */
	if (argc<2) { 
		usage();
		return (1);
	}

	/* 					*/
	/* Check for other command-line options */
	/*                                      */
	/*** User asked for help ***/
	if (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help")) {
		usage();
		return (0);
	}

	/*** User wants version information ***/
	if (!strcmp(argv[1], "--version")) {
		printf("wrap - version %s\n", VERSION);
		return (0);
	}

	/*                          */
	/* Get the number specified */
	/*                          */
	strcpy(sWrapAt, argv[1]);

	/*                                              */
	/* Be sure the first argument starts with a "-" */
	/*                                              */
	if (sWrapAt[0] != '-') {
		printf("ERROR: Invalid option %s!\n", sWrapAt);
		usage();
		return (1);
	}

	/*                                                                      */
	/* Get rid of the "-" in the first argument by shifting characters left */
	/*                                                                      */
	for (x=0; x<strlen(sWrapAt)-1; ++x) {
		sWrapAt[x]=sWrapAt[x+1];
	}
	sWrapAt[x]='\0';

	/*                                  */
	/* Convert the string to an integer */
	/*                                  */
	iWrapAt=atoi(sWrapAt);
	if (iWrapAt==0) {		/*** This also catches user errors like "-asdf" for a number ***/
		printf("ERROR: Can't wrap at 0 characters!\n");
		usage();
		return (1);
	}

	/*                                                                      */
	/* If there is a filename specified, open the file, otherwise use stdin */
	/*                                                                      */
	if (argc>2) {		/*** There is a filename argument present...try to use it ***/
		filename=fopen(argv[2], "r");
		if (filename == NULL) {
			printf("ERROR: Couldn't open file %s!\n", argv[2]);
			return (1);
		}
	}
	else {			/*** User did not specify a file...assume stdin ***/
		filename=stdin;
	}

	/*                                   */
	/* Wrap the file, and echo to stdout */
	/*                                   */
	/*** Process each line ***/
	while (fgets(InString, MAX_STR_LEN, filename) != NULL) {

		/*** First, discard the embedded newline added by fgets ***/
		InString[strlen(InString)-1]='\0';
		
		/*** Record the index of the last character in the string ***/
		LastPos=strlen(InString)-1;

		/*** Now, print the first iWrapAt characters, then the next, and so on ***/
		/*** until there are fewer characters remaining than the user wants on a line ***/
		CurPos=0;		/* We're starting at the first character on the line ***/
		Offset=1;		/* First set of iWrapAt characters... ***/
		while (LastPos-CurPos > iWrapAt) {
			for ( ; CurPos<iWrapAt*Offset; ++CurPos) {
				putchar(InString[CurPos]);  	/*** echo iWrapAt chars to stdout ***/
			}
			printf("\n");				/*** echo a newline to stdout ***/
			++Offset;				/*** next set of iWrapAt chars...***/
		}
		
		/*** Finally, print the last portion of the string and a newline ***/
		for ( ; CurPos<=LastPos; ++CurPos) {
			putchar(InString[CurPos]);
		}
		printf("\n");
	} /*** Get the next line of input ***/

	/*                                         */
	/* Close the file if we're not using stdin */
	/*                                         */
	if (filename != stdin) {
		if (fclose(filename)) {
			printf("ERROR: Couldn't close file %s!\n", argv[2]);
			return (1);
		}
	}
	
	/*                       */
	/* Successful completion */
	/*                       */
	return (0);
}

void usage(void)
{
	printf("Usage: wrap [OPTION]... -NUMBER [FILE]\n");
	printf("Echo the contents of FILE to stdout, wrapping at NUMBER characters per\n");
	printf("line. If NUMBER is bigger than the length of the input line, the line\n");
	printf("is not wrapped. If FILE is omitted, input is taken from stdin.\n\n");
	printf("  -h, --help\t\tdisplay this help and exit\n");
	printf("      --version\t\tprint version information and exit\n");
}
